# -*- coding: utf-8 -*-
'''*****************************************************************************
***  This file is part of HgWeb Manager.                                     ***
***                                                                          ***
***  Copyright (C) 2014                                                      ***
***  CodeGNU Solutions <Licensing _AT_ CodeGNU _DOT_ com>                    ***
***                                                                          ***
***  This program is free software: you can redistribute it and/or modify it ***
***  under the terms of the GNU Affero General Public License as published   ***
***  by the Free Software Foundation, either version 3 of the License, or    ***
***  (at your option) any later version.                                     ***
***                                                                          ***
***  This program is distributed in the hope that it will be useful, but     ***
***  WITHOUT ANY WARRANTY; without even the implied warranty of              ***
***  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    ***
***  See the GNU Affero General Public License for more details.             ***
***                                                                          ***
***  You should have received a copy of the GNU Affero General Public License***
***  along with this program. If not, see <http://www.gnu.org/licenses/>.    ***
*****************************************************************************'''

from manager import settings


class Repository(object):
	""" Describes the properties of a repository managed by HWM and served up
	by HgWeb.
	"""

	# The directory name of the managed repository.
	__storageName = None
	# The displayed web-site name of the managed repository.
	__displayName = None
	# The displayed web-site description of the managed repository.
	__description = None
	# The displayed web-site contact information of the managed repository.
	__contact = None
	# The Mercurial repository interface.
	__repo = None
	# The collection of plug-ins enabled for a repository.
	__plugIns = []

	@property
	def StorageName(self):
		""" Gets the name of the directory where the managed repository is located. """
		return self.__storageName

	@property
	def DisplayName(self):
		""" Gets the name to display for the managed repository. """
		return self.__displayName

	@DisplayName.setter
	def DisplayName(self, value):
		""" Sets the name of the managed repository to display. """
		self.__displayName = value

	@property
	def Description(self):
		""" Gets the description of the managed repository. """
		return self.__description

	@Description.setter
	def Description(self, value):
		""" Sets the description of the managed repository to display. """
		self.__description = value

	@property
	def Contact(self):
		""" Gets the contact point for the managed repository. """
		return self.__contact

	@Contact.setter
	def Contact(self, value):
		""" Sets the contact point for the managed repository. """
		self.__contact = value

	@property
	def PlugIns(self):
		""" Gets the collection of enabled plug-ins for the managed repository. """
		return self.__plugIns

	def __init__(self, repoName=None):
		""" Initialises the object with the values from a managed repository

		@param[in] repoName		The storage name of the managed repository
								to load.
		"""
		from mercurial import ui, hg, error
		import os

		self.__storageName = repoName
		if repoName:
			try:
				self.__repo = hg.repository(ui.ui(), settings.RepositoryPath + os.sep + self.__storageName)
				self.__displayName = self.__repo.ui.config('web', 'name', default=None)
				self.__description = self.__repo.ui.config('web', 'description', default=None)
				self.__contact = self.__repo.ui.config('web', 'contact', default=None)
				plugInList = self.__repo.ui.configitems('extensions')
				for plugIn in plugInList:
					self.__plugIns.append(RepositoryPlugin(plugIn[0], plugIn[1]))
			except error.RepoError:
				pass

	def __eq__(self, other):
		""" Determines if two repositories are the same repository.

		@param[in] other	The repository on the right-hand-side of the
							equality operator to compare against.

		@return Gives true when both repositories represent the same one,
		else-wise gives false.
		"""
		return self.__storageName == other.__storageName

	def save(self):
		""" Commits the repository detail properties to the repository
		configuration file.

		@throws IOError		When the configuration file for the repository
							fails to be written.
		"""
		import os

		with open(settings.RepositoryPath + os.sep + self.__storageName + os.sep + '.hg' + os.sep + 'hgrc', 'w') as hgrc:
			hgrc.write('[extensions]\n')
			for plugIn in self.PlugIns:
				if plugIn.scriptFile is None:
					hgrc.write(plugIn.name + ' =\n')
				else:
					hgrc.write(plugIn.name + ' = ' + plugIn.scriptFile + '\n')

			hgrc.write('\n[web]\n')
			if self.DisplayName is None:
				hgrc.write('name = ' + self.StorageName + '\n')
			else:
				hgrc.write('name = ' + self.DisplayName + '\n')

			if self.Description is not None:
				hgrc.write('description = ' + self.Description + '\n')

			if self.Contact is not None:
				hgrc.write('contact = ' + self.Contact + '\n')
		# Needed for when repositories are re-named.
		self = Repository(self.StorageName)

	def __str__(self):
		""" Converts the object contents into a human-friendly string. """
		repo = "%s\n\tDisplay: %s\n\tDescription: %s\n\tContact: %s" % (self.StorageName, self.DisplayName, self.Description, self.Contact)
		if len(self.PlugIns) > 0:
			repo = repo + "\n\tPlug-ins:"
			for plugIn in self.PlugIns:
				repo = repo + "\n\t\t%s" % plugIn
		return repo

	def addPlugIn(self, plugIn):
		""" Makes a plug-in in a repository enabled.

		@param[in] plugIn	The plug-in that is to be enabled.

		@post The object now has the supplied plug-in as being enabled.

		@throws ValueError		When the supplied plug-in is already enabled.
		"""
		if plugIn not in self.PlugIns:
			self.__plugIns.append(plugIn)
		else:
			raise ValueError("The plug-in '%s' is already enabled." % plugIn.name)

	def removePlugIn(self, named):
		""" Makes a plug-in in a repository disabled.

		@param[in] named	The name of the plug-in that is to be disabled.

		@post The object no longer has the supplied plug-in as being enabled.

		@throws ValueError		When the supplied plug-in is already disabled.
		"""
		for plugIn in self.PlugIns:
			if named == plugIn.name:
				self.__plugIns.remove(plugIn)
				return
		raise ValueError("The plug-in '%s' is already disabled." % named)


class RepositoryPlugin(object):
	__name = None
	__scriptFile = None

	@property
	def name(self):
		return self.__name

	@property
	def scriptFile(self):
		return self.__scriptFile

	def __init__(self, name, scriptFile=None):
		import os

		self.__name = name
		if (scriptFile is not None and len(scriptFile) > 0):
			if os.path.exists(scriptFile):
				self.__scriptFile = scriptFile
			else:
				raise ValueError("The plug-in file '%s' was not found." % scriptFile)

	def __eq__(self, other):
		""" Determines if two repository plug-ins are the same.

		@param[in] other	The plug-in on the right-hand-side of the
							equality operator to compare against.

		@return Gives true when both plug-ins represent the same one,
		else-wise gives false.
		"""
		return self.name == other.name

	def __str__(self):
		""" Converts the object contents into a human-friendly string. """
		if self.scriptFile is None:
			return self.name
		else:
			return "%s -> %s" % (self.name, self.scriptFile)


class ManagedCollection(object):
	""" A container of all the managed repositories. """

	# All the managed repositories
	__repositories = []

	@property
	def Repositories(self):
		"""A collection of all managed repositories.

		@return An array of the managed repositories.
		"""
		return self.__repositories

	def __init__(self):
		""" Initialises the container with all the managed repositories.

		@throws RuntimeError	When the HgWeb configuration cannot be read.
		"""
		from ConfigParser import SafeConfigParser
		import os

		parser = SafeConfigParser()
		if not parser.read(settings.HgWebPath + os.sep + 'hgweb.config'):
			raise RuntimeError("Failed to read HgWeb configuration.")

		for name, path in parser.items('paths'):
			self.__repositories.append(Repository(name))

	def __str__(self):
		""" Converts the object contents into a human-friendly string. """
		collection = ""
		for repository in self.Repositories:
			collection = collection + str(repository) + '\n'
		collection.rstrip('\n')
		return collection
